@@ -954,6 +954,10 @@ class Tensor(torch.Tensor):
                 self.transpose_(s[1] + i + 1, self.ndim - i - 1)
         return self
 
+    def __get__(self):
+        if not self.init: return self.special_from_()
+        else: return self
+
     def __matmul__(self, other, **kwargs):
         if isinstance(other, torch.Tensor) and self.has_special or isinstance(other, Tensor) and other.has_special:
             a, b = self.shape[:-2] ^ other.shape[:-2]
@@ -1082,7 +1086,7 @@ class Tensor(torch.Tensor):
     def __torch_function_resizing_func__(cls, func, types, args=(), kwargs=None):
         "FOR: reshape view zeros ones rand randn"
         dims = Size(args[1:])
-        args = args[:1] + (dims,)
+        args = args[:1] + tuple(dims)
 
         with torch._C.DisableTorchFunction():
             ret = super().__torch_function__(func, types, args, kwargs)
